import typing
import contextlib

import matplotlib as mpl

import pympljstyle.base


def apply_style(
    journal_name: str,
    width: str,
    height: str = "1.5 widths",
    content_type: str = "combination",
    with_opinionated_defaults: bool = True,
) -> contextlib.AbstractContextManager[None]:

    style = get_style(
        journal_name=journal_name,
        width=width,
        height=height,
        content_type=content_type,
        with_opinionated_defaults=with_opinionated_defaults,
    )

    context = mpl.rc_context(rc=style)

    return context


def get_style(
    journal_name: str,
    width: str,
    height: str = "1.5 widths",
    content_type: str = "combination",
    with_opinionated_defaults: bool = True,
) -> dict[str, typing.Any]:

    JournalStyle = pympljstyle.base.registry[journal_name]  # noqa: N806

    journal_style = JournalStyle(width=width, height=height, content_type=content_type)

    if with_opinionated_defaults:
        rc_params = pympljstyle.base.get_defaults()
    else:
        rc_params = {}

    rc_params |= journal_style.rcParams

    return rc_params


def get_registered_journals() -> tuple[str, ...]:

    return tuple(
        JournalStyle.info() for JournalStyle in pympljstyle.base.registry.values()
    )



import matplotlib.axes


def apply_dropped_spines(
    ax: matplotlib.axes._axes.Axes,
    amount_pt: int | float = 10,
) -> None:

    for spine in ax.spines.values():
        spine.set_position(("outward", amount_pt))



import matplotlib.pyplot as plt

import pympljstyle


def test_initial_reg() -> None:
    assert len(pympljstyle.get_registered_journals()) > 0


def test_rc_params() -> None:

    style = pympljstyle.get_style(
        journal_name="cortex",
        width="1 col",
    )

    assert style != {}


def test_rc_params_set() -> None:

    start_rc_params = dict(plt.rcParams)

    with pympljstyle.apply_style(
        journal_name="cortex",
        width="1 col",
    ):

        curr_rc_params = dict(plt.rcParams)

        assert curr_rc_params != start_rc_params

    end_rc_params = dict(plt.rcParams)

    assert start_rc_params == end_rc_params



import pympljstyle


def test_add_custom() -> None:

    assert "custom_test" not in pympljstyle.registry

    @pympljstyle.add_journal
    class CustomTest(pympljstyle.BaseJournal):

        name = "custom_test"
        journal = "Custom Test"
        custom_units = ()

        def add_custom_settings(self) -> None:
            pass

        def add_custom_units(self) -> None:
            pass

    assert "custom_test" in pympljstyle.registry


def test_all_styles() -> None:

    for journal_name in pympljstyle.registry:

        if journal_name == "custom_test":
            continue

        _ = pympljstyle.get_style(
            journal_name=journal_name,
            width="1cm",
        )



import typing

import pint

import pympljstyle.base


# indicate the attributes available in the Base journal
class HasBaseProtocol(typing.Protocol):
    @property
    def _rc_params(self) -> dict[str, typing.Any]: ...
    @property
    def _content_type(self) -> str: ...
    @property
    def _ureg(self) -> pint.registry.UnitRegistry: ...


# this can be used for general Elsevier journals
class ElsevierMixin:

    def add_custom_settings(self: HasBaseProtocol) -> None:

        dpi = {
            "halftone": 300,
            "combination": 500,
            "line": 1000,
        }

        self._rc_params["savefig.dpi"] = dpi[self._content_type]

    def add_custom_units(self: HasBaseProtocol) -> None:
        self._ureg.define("column = 100 mm; offset: -10 = col")


@pympljstyle.base.add_journal
class Cortex(ElsevierMixin, pympljstyle.base.BaseJournal):

    name = "cortex"
    journal_name = "Cortex"
    custom_units = ("1 column", "1.5 columns", "2 columns")


@pympljstyle.base.add_journal
class JNeurophys(pympljstyle.base.BaseJournal):

    name = "j_neurophys"
    journal_name = "Journal of Neurophysiology"
    custom_units = ("single", "double", "full")

    def add_custom_settings(self) -> None:

        dpi = {
            "halftone": 300,
            "combination": 600,
            "line": 1200,
        }

        self._rc_params["savefig.dpi"] = dpi[self._content_type]

    def add_custom_units(self) -> None:
        self._ureg.define("single = 8.9 cm")
        self._ureg.define("double = 12 cm")
        self._ureg.define("full = 18 cm")



import typing
import abc

import pint


class BaseJournal(abc.ABC):

    def __init__(
        self,
        width: str,
        height: str,
        content_type: str = "combination",
    ) -> None:

        self._width_raw = width
        self._height_raw = height

        if content_type not in ["halftone", "combination", "line"]:
            raise ValueError("Unknown `content_type`")

        self._content_type = content_type

        self._ureg = pint.UnitRegistry(
            cache_folder=":auto:",
            autoconvert_offset_to_baseunit=True,
        )

        self._rc_params: dict[str, typing.Any] = {}

    @abc.abstractmethod
    def add_custom_units(self) -> None:
        pass

    @abc.abstractmethod
    def add_custom_settings(self) -> None:
        pass

    @property
    @abc.abstractmethod
    def name(self) -> str:
        return self.name

    @property
    @abc.abstractmethod
    def journal_name(self) -> str:
        return self.journal_name

    @property
    @abc.abstractmethod
    def custom_units(self) -> tuple[str, ...]:
        return self.custom_units

    @classmethod
    def info(cls) -> str:  # noqa: ANN102
        info_str = f'"{cls.name}": {cls.journal_name}'
        # explicit type verification because mypy gets confused by properties
        custom_units = cls.custom_units
        assert isinstance(custom_units, tuple)
        if len(custom_units) > 0:
            info_str += f' (custom units: {", ".join(custom_units)})'
        return info_str

    def get_size(self, units: str = "inches") -> tuple[float, float]:

        width = self._ureg.Quantity(value=self._width_raw)

        self._ureg.define(f"width = 1 * {width.to('mm')} = w")

        height = self._ureg.Quantity(value=self._height_raw)

        return tuple(dim.to(other=units).magnitude for dim in (width, height))

    @property
    def rcParams(self) -> dict[str, typing.Any]:  # noqa: N802

        self.add_custom_settings()
        self.add_custom_units()

        figure_size_entry = {
            "figure.figsize": self.get_size(units="inches"),
        }

        return self._rc_params | figure_size_entry


registry: dict[str, type[BaseJournal]] = {}

T = typing.TypeVar("T", bound=BaseJournal)


def add_journal(cls: type[T]) -> type[T]:
    registry[str(cls.name)] = cls
    return cls


def get_defaults() -> dict[str, typing.Any]:

    defaults = {
        # font sizes
        "font.size": 9,
        "axes.titlesize": 9,
        "xtick.labelsize": 8,
        "ytick.labelsize": 8,
        "legend.fontsize": 7,
        "legend.title_fontsize": 7,
        # other params
        "lines.linewidth": 1,
        "axes.spines.right": False,
        "axes.spines.top": False,
        "figure.frameon": False,
        "font.sans-serif": [
            "Arial",
            "Nimbus Sans",
            "Nimbus Sans L",
            "Helvetica",
        ],
    }

    return defaults



__version__ = "0.2"

import pympljstyle.journals  # noqa: F401

from .base import registry, BaseJournal, add_journal

from .interface import apply_style, get_style, get_registered_journals

__all__ = (
    "apply_style",
    "get_style",
    "registry",
    "get_registered_journals",
    "BaseJournal",
    "add_journal",
)
